# -*- coding: utf-8 -*-
import datetime
import re
try: # On utilise cStringIO si possible
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

import django.forms as forms
import django.http as http
import django.views.generic as generic_views
from django.conf import settings
from django.contrib import messages
from django.core.urlresolvers import reverse
from django.shortcuts import render

import contacts.models as contacts_models
import contacts.utils as contacts_utils

import model_walk

### admin_actions
def gen_etiquettes(model_admin, request, queryset):
    ""
    liste_contacts = []
    for contact in queryset:
        liste_contacts.append(contact)
        for pers_associees in contact.liste_contacts_associes():
            if pers_associees.courrier_structure:
                liste_contacts.append(pers_associees)
    if len(liste_contacts) == 0:
        raise http.Http404(u"Aucun contact n'a été trouvé !")
    # Génération et renvoi du document
    buffer = StringIO()
    pdfdoc = contacts_utils.EtiquettesPDF(buffer)
    pdfdoc.build(liste_contacts)
    response = http.HttpResponse(mimetype='application/pdf')
    response['Content-Disposition'] = 'attachment; filename=etiquettes.pdf'
    response.write(buffer.getvalue())
    buffer.close()
    return response


gen_etiquettes.short_description = u'Générer des étiquettes'


def _gen_mailto(queryset, sympa=False):
    "shared code between full views and admin_action"
    adresses = []
    adresses_nospam = []
    sans_adresse = []
    for contact in queryset:
        liste_adr = contact.get_courriels()
        if liste_adr:
            # on collecte l'adresse si elle n'est pas déjà sélectionnée
            adrs = {}
            for nom, email in liste_adr:
                if not adrs.has_key(email):
                    adrs[email] = True
                    ##
                    ext_mail = re.sub(
                        '  *',' ', email.replace(',',' ').replace(';',' ')
                        .strip()).split(' ')
                    for sub_mail in ext_mail:
                        if nom:
                            dest = (
                                (u'{1} "{0}"' if sympa else u'"{0}" <{1}>')
                                .format(nom.strip(), sub_mail))
                        else:
                            dest = email
                        adresses.append(dest)
        else:
            sans_adresse.append(
                (unicode(contact), contact.get_absolute_url()))
    return (adresses, sans_adresse)


def gen_mailto_gen(model_admin, request, queryset, sympa=False):
    "suitable for admin action"
    dest, no_adr = _gen_mailto(queryset, sympa)
    mc = model_admin.model
    url_changelist = reverse("admin:%s_%s_changelist" %
                             (mc._meta.app_label, mc.__name__.lower()))
    context = {
        'sympa': sympa,
        'destinataires': dest,
        'sans_adresse': no_adr,
        'nom_objet': {'singulier': mc._meta.verbose_name,
                      'pluriel': mc._meta.verbose_name_plural},
        'css_app': settings.STATIC_URL + "css/contacts.css",
        'url_retour': url_changelist,
        }
    return render(request, "admin/contacts/mailto.html", context)


def gen_mailto(model_admin, request, queryset):
    return gen_mailto_gen(model_admin, request, queryset)


gen_mailto.short_description = u'Envoyer un courriel'


def gen_mailto_sympa(model_admin, request, queryset):
    return gen_mailto_gen(model_admin, request, queryset, sympa=True)


gen_mailto_sympa.short_description = u'Génerer une liste pour Sympa'


def _gen_csv(
    request, model, queryset, incl_p_assoc=False, nom_fichier="export.csv",
    msg_aucun_objet=u"Le panier ne contient pas d'objet correspondant",
    classe_export=None):
    object_list = []
    for obj in queryset:
        object_list.append(obj)
        if incl_p_assoc:
            object_list.extend(
                [passoc for passoc in obj.liste_contacts_associes()
                 if passoc.courrier_structure])
    if len(object_list) == 0:
        messages.error(request, msg_aucun_objet)
        return http.HttpResponseRedirect(
            reverse('admin:%s_%s_changelist' %
                    (model._meta.app_label, model.__name__.lower())))
    buffer = StringIO()
    csvdoc = classe_export(buffer)
    csvdoc.generer(object_list)
    response = http.HttpResponse(mimetype='text/csv')
    response['Content-Disposition'] = 'attachment; filename='+nom_fichier
    response.write(buffer.getvalue())
    buffer.close()
    return response


def gen_csv_complet(model_admin, request, queryset):
    ""
    return _gen_csv(
        request, model_admin.model, queryset, nom_fichier="export_donnees.csv",
        classe_export=contacts_utils.ExportCSVComplet)
                    

gen_csv_complet.short_description = u'Exporter les données en CSV'


def gen_csv_assolement(model_admin, request, queryset):
    ""
    assolement = contacts_models.Assolement.objects.filter(
        donneesbio__in = queryset)
    return _gen_csv(
        request, contacts_models.Assolement, assolement, 
        nom_fichier="export_assolements_donnees.csv",
        classe_export=contacts_utils.ExportCSVComplet)
                    

gen_csv_assolement.short_description = u'Exporter les assolements des données en CSV'


def gen_csv_commercialisation(model_admin, request, queryset):
    ""
    commercialisation = contacts_models.Commercialisation.objects.filter(
        donneesbio__in = queryset)
    return _gen_csv(
        request, contacts_models.Commercialisation, commercialisation, 
        nom_fichier="export_commercialisations_donnees.csv",
        classe_export=contacts_utils.ExportCSVComplet)
                    

gen_csv_commercialisation.short_description = u'Exporter les commercialisations des données en CSV'


def gen_csv_transformation(model_admin, request, queryset):
    ""
    transformation = contacts_models.Transformation.objects.filter(
        donneesbio__in = queryset)
    return _gen_csv(
        request, contacts_models.Transformation, transformation, 
        nom_fichier="export_transformations_donnees.csv",
        classe_export=contacts_utils.ExportCSVComplet)
                    

gen_csv_transformation.short_description = u'Exporter les transformations des données en CSV'


def gen_csv_publipostage(model_admin, request, queryset):
    ""
    return _gen_csv(
        request, model_admin.model, queryset,
        nom_fichier="publipostage_contacts.csv",
        classe_export=contacts_utils.ExportCSVPublipostage)

gen_csv_publipostage.short_description = u'Exporter pour publipostage'


class ToggleValDonneesBio(
    generic_views.detail.SingleObjectMixin, generic_views.RedirectView):
    action = 'toggle'
    model = contacts_models.DonneesBio

    def get(self, request, *args, **kwargs):
        self.obj = self.get_object()
        today = datetime.date.today()
        self.obj.date_validation = {
            'toggle': None if self.obj.date_validation else today,
            'validate': today,
            'unvalidate': None,
            }.get(self.action)
        self.obj.save()
        messages.info(request, u"Les données ont été {0}validées".format(
                u"" if self.obj.date_validation else u"dé - "))
        return super(ToggleValDonneesBio, self).get(request, *args, **kwargs)
        
    def get_redirect_url(self, *args, **kwargs):
        return reverse('admin:contacts_donneesbio_change', args=[self.obj.id])

### ---------------------------------------------------------------------------
# Vue créant un nouvel objet DonneesBio pour un agriculteur bio et redirigant
# vers le formulaire de modification de cet objet.
# (évite l'utilisation de la longue liste des agriculteurs bio).

def adapt_fk_field(list_fk, init_dict):
    "transform foreign key field label by appending '_id' at end"
    for item in list_fk:
        if item in init_dict:
            init_dict[item + '_id'] = init_dict[item]
            del init_dict[item]
    return init_dict

def split_m2m_field(list_m2m, init_dict):
    m2m_dict = {}
    for item in list_m2m:
        if item in init_dict:
            m2m_dict[item] = init_dict[item]
            del init_dict[item]
    return (init_dict, m2m_dict)

class CreeDonneesBio(generic_views.base.RedirectView):
    ""
    option_duplication = True
    def get(self, request, *args, **kwargs):
        ""
        agri = contacts_models.Structure.objects.get(
            pk=self.kwargs['pk'])
        this_year = datetime.date.today().year
        last_donneesbio = agri.last_donneesbio()
        if last_donneesbio:
            if last_donneesbio.annee == this_year:
                self.donneesbio = last_donneesbio
                messages.warning(
                    self.request, u"Les données de l'année existe déjà"
                    u" redirection vers les données {0}".format(this_year))
            else:
                if self.option_duplication:
                    # preremplir avec last_donneesbio
                    init_dict = forms.model_to_dict(last_donneesbio)
                    del init_dict['id']
                    init_dict.update({
                            'date_collecte': None,
                            'date_validation': None,
                            'date_modification': None,
                            'annee': this_year,
                            'remarques': "Attention copie de %s\n%s" % (
                                last_donneesbio.annee,
                                init_dict['remarques'] or ''),
                            })
                    list_fk = model_walk.get_fk_name_from_model(
                        last_donneesbio)
                    adapt_fk_field(list_fk, init_dict)
                    list_m2m = model_walk.get_m2m_name_from_model(
                        last_donneesbio)
                    init_dict, m2m_dict = split_m2m_field(list_m2m, init_dict)
                    self.donneesbio = (
                        contacts_models.DonneesBio.objects.create(
                            **init_dict))
                    for key, val in m2m_dict.iteritems():
                        setattr(self.donneesbio, key, val)
                    self.creeInlineDonneesBio(
                    last_donneesbio, contacts_models.Commercialisation)
                    self.creeInlineDonneesBio(
                    last_donneesbio, contacts_models.Assolement)
                    self.creeInlineDonneesBio(
                    last_donneesbio, contacts_models.Transformation)
                else:
                    # creation ex-nihilo en attendant que la fonction
                    # soit prete
                    self.donneesbio = (
                        contacts_models.DonneesBio.objects.create(
                            agriculteur_bio=agri))
        else:
            # creation ex-nihilo
            self.donneesbio = (
                contacts_models.DonneesBio.objects.create(
                    agriculteur_bio=agri))

        return super(CreeDonneesBio, self).get(
            self, request, *args, **kwargs)

    def creeInlineDonneesBio(self, last_donneesbio, model):
        list_instance = model.objects.filter(
        donneesbio__id=last_donneesbio.id)
        for instance in list_instance:
            init_dict = forms.model_to_dict(instance)
            del init_dict['id']
            init_dict.update({
                'donneesbio': self.donneesbio.id
                })
            list_fk = model_walk.get_fk_name_from_model(instance)
            adapt_fk_field(list_fk, init_dict)
            list_m2m = model_walk.get_m2m_name_from_model(instance)
            init_dict, m2m_dict = split_m2m_field(list_m2m, init_dict)
            self.instance = (model.objects.create(**init_dict))
            for key, val in m2m_dict.iteritems():
                setattr(self.instance, key, val)
                
    def get_redirect_url(self, **kwargs):
        """
        Retourne l'URL de la vue de modification de l'objet DonneesBio
        récemment créé.
        """
        return reverse(
            'admin:contacts_donneesbio_change', args=[self.donneesbio.pk])

# EOF
