# -*- coding: utf-8 -*-

"""
helper function for datamigration
"""

from django.db import IntegrityError
from django.db.models import ObjectDoesNotExist

#  qual_compl / qualificatifs_complementaires
AGRI_FIELD = (
    'visibilite',
    'type_activite',
    'quota_laitier',
    'laiterie',
    'nb_vaches_allaitantes',
    'sau',
    'sau_bio',
    'sau_eau',
    'sau_eau_bio',
    'parc_naturel_regional',
    )

AGRIBIO_FIELD = (
    'date_envoi_courrier',
    'premiere_adhesion',
    'renouvellement_adhesion',
    )

def copy_agri_struct_field(structure, dryrun=True, south=False):
    # traitement agriculteur
    try:
        agri = structure.agriculteur
    except ObjectDoesNotExist, err:
        return
    for cf in AGRI_FIELD:
        setattr(structure, cf, getattr(agri, 'leg_' + cf))
    # attention n'est pas un dryrun !
    structure.qual_compl = agri.qualificatifs_complementaires.all()
    # traitement agriculteurbio
    try:
        agribio = agri.agriculteurbio
    except ObjectDoesNotExist, err:
        agribio = None
    if agribio:
        for cf in AGRIBIO_FIELD:
            setattr(structure, cf, getattr(agribio, 'leg_' + cf))
    if not dryrun:
        if south:
            structure.save()
        else:
            structure.save(rawsave=True)


def check_agri_struct_field(structure, verbose=False):
    err_cpt = 0
    try:
        agri = structure.agriculteur
    except ObjectDoesNotExist, err:
        return err_cpt
    # traitement agriculteur
    for cf in AGRI_FIELD:
        nvx = getattr(structure, cf)
        leg = getattr(agri, 'leg_' + cf)
        if nvx != leg:
            err_cpt += 1
            if verbose:
                print "struct:{0} field:{1} leg[{2}]nvx[{3}]".format(
                    structure.id, cf, leg, nvx)
    if ([elm.id for elm in structure.qual_compl.order_by('id')] !=
        [elm.id for elm in agri.qualificatifs_complementaires.order_by('id')]):
        err_cpt += 1
        if verbose:
            print "struct:{0} field:{1} leg[{2}]nvx[{3}]".format(
                structure.id, 'qual_compl',
                ",".join(
                    [str(elm.id) for elm in
                     agri.qualificatifs_complementaires
                     .order_by('id')]),
                ",".join(
                    [str(elm.id) for elm in
                     structure.qual_compl.order_by('id')]))
    # traitement agriculteurbio
    try:
        agribio = agri.agriculteurbio
    except ObjectDoesNotExist, err:
        return err_cpt
    for cf in AGRIBIO_FIELD:
        nvx = getattr(structure, cf)
        leg = getattr(agribio, 'leg_' + cf)
        if nvx != leg:
            err_cpt += 1
            if verbose:
                print "struct:{0} field:{1} leg[{2}]nvx[{3}]".format(
                    structure.id, cf, leg, nvx)
    return err_cpt


def run_copier(struct_qs, dryrun=True, south=False):
    for struct in struct_qs:
        err_cpt = copy_agri_struct_field(struct, dryrun, south)


def run_checker(struct_qs, verbose=False, south=False):
    gen_err_cpt = [0, 0]
    err_cpt = 0
    for struct in struct_qs:
        err_cpt = check_agri_struct_field(struct, verbose)
        gen_err_cpt[1] += err_cpt
        if err_cpt > 0:
            gen_err_cpt[0] += 1
            if verbose:
                print "Check structure id: {0} ({1} err)".format(
                    struct.id, err_cpt)
    if south and gen_err_cpt[0] > 0:
        raise IntegrityError("{0} errors".format(gen_err_cpt[0]))
    return gen_err_cpt

# EOF
