# -*- coding: utf-8 -*-
from django.contrib.auth import models as am
from django.core.exceptions import ValidationError
from django.test import TestCase

#from contacts import models as cm
from contacts.tests.utils import mfty, cfty
from references import models as rm

from django.contrib import messages
from django.test.client import Client

from contacts import settings_messages as MSG

class ValAdminStructure(TestCase):

    def setUp(self):
        self.messagePersonne = MSG.AVERTISSEMENT_MESSAGE["message_personne_manquante"]
        self.messageProjet = MSG.AVERTISSEMENT_MESSAGE["message_projet_bio_manquant"]
        self.usr1 = am.User.objects.create_superuser('usr1', '', 'mdp')
        self.usr1.save()
        self.client.login(username = self.usr1.username, password = 'mdp')
        self.fjur = rm.FormeJuridique()
        self.fjur.save()
        self.activite = rm.TypeActivite()
        self.activite.save()
        self.data = {
            'adhesiongabnor_set-TOTAL_FORMS': u'0', 
            'adhesiongabnor_set-INITIAL_FORMS': u'0', 
            'adhesiongabnor_set-MAX_NUM_FORMS': u'',
                                                                
            'personne_set-TOTAL_FORMS': u'0', 
            'personne_set-INITIAL_FORMS': u'0', 
            'personne_set-MAX_NUM_FORMS': u'',
            
            'projetbio_set-TOTAL_FORMS': u'0', 
            'projetbio_set-INITIAL_FORMS': u'0', 
            'projetbio_set-MAX_NUM_FORMS': u'', 
            
            'rdvpersonnemorale_set-TOTAL_FORMS': u'0', 
            'rdvpersonnemorale_set-INITIAL_FORMS': u'0', 
            'rdvpersonnemorale_set-MAX_NUM_FORMS': u'', 
            
            'f_juridique': str(self.fjur.id), 
            'designation': 'test',
            'visibilite': 'personne',
            }
        self.data_projet = {
            'projetbio_set-TOTAL_FORMS' : u'1',
        
            'projetbio_set-0-pond_savoirfaire' : u'-1',
            'projetbio_set-0-pond_motivation' : u'0',
            'projetbio_set-0-pond_entourage' : u'0',
            'projetbio_set-0-pond_foncier' : u'0',
            'projetbio_set-0-pond_materiel' : u'0',
            'projetbio_set-0-pond_reglementation' : u'0',
            'projetbio_set-0-pond_commercialisation' : u'0',
            'projetbio_set-0-pond_financement' : u'0',
            'projetbio_set-0-pond_maindoeuvre' : u'0',
            
            'projetbio_set-0-installation_bio' : True
            }
        self.data_personne = {
            'personne_set-TOTAL_FORMS' : u'1',
            'personne_set-0-nom' : 'testin',
            'personne_set-0-prenom' : 'teston',
            'personne_set-0-date_naissance' : '2000-01-01'
            }
        
    def test_structure_personne(self):
        """Une structure de type agri ou agri bio devrait 
        avoir une personne physique affiliée"""
        
        # Structure agri sans personne affiliée avec avertissement
        data = self.data.copy()
        data['type_pm'] = u'1'
        data['type_activite_pri'] = str(self.activite.id)
        response = self.client.post('/private/contacts/structure/add/', 
                                    data,
                                    follow=True)
        msg2 = list()
        for message in list(response.context['messages']):
            msg2.append(message.message)       
        self.assertNotEqual(len(msg2), 0, "Erreur lors de l'enregistrement")
        self.assertIn(self.messagePersonne, msg2)
        
        # Structure agri avec personne affiliée sans avertissement
        data.update(self.data_personne)
        response = self.client.post('/private/contacts/structure/add/', 
                                    data,
                                    follow=True)
        msg2 = list()
        for message in list(response.context['messages']):
            msg2.append(message.message)       
        self.assertNotEqual(len(msg2), 0, "Erreur lors de l'enregistrement")
        self.assertNotIn(self.messagePersonne, msg2)
        
        # Structure simple sans personne affiliée sans avertissement
        data = self.data.copy()
        data['type_pm'] = 0
        response = self.client.post('/private/contacts/structure/add/', 
                                    data,
                                    follow=True)
        msg2 = list()
        for message in list(response.context['messages']):
            msg2.append(message.message)
        self.assertNotEqual(len(msg2), 0, "Erreur lors de l'enregistrement")
        self.assertNotIn(self.messagePersonne, msg2)
        
    def test_structure_projet(self):
        """ Une structure de type agri bio devrait 
        avoir un projet bio avec date d'engagement OC"""
        
        # Structure agri sans projet sans avertissement
        data = self.data.copy()
        data['type_pm'] = 1
        data['type_activite_pri'] = self.activite.id
        response = self.client.post('/private/contacts/structure/add/', 
                                    data,
                                    follow=True)
        msg2 = list()
        for message in list(response.context['messages']):
            msg2.append(message.message)
        self.assertNotEqual(len(msg2), 0, "Erreur lors de l'enregistrement")
        self.assertNotIn(self.messageProjet, msg2)
        
        # Structure agri bio sans projet avec avertissement
        data = self.data.copy()
        data['type_pm'] = 2
        data['type_activite'] = self.activite.id
        response = self.client.post('/private/contacts/structure/add/', 
                                    data,
                                    follow=True)
        msg2 = list()
        for message in list(response.context['messages']):
            msg2.append(message.message)
        self.assertNotEqual(len(msg2), 0, "Erreur lors de l'enregistrement")
        self.assertIn(self.messageProjet, msg2)
        
        # Structure agri bio avec projet sans date avec avertissement
        data.update(self.data_projet)
        response = self.client.post('/private/contacts/structure/add/', 
                                    data,
                                    follow=True)
        msg2 = list()
        for message in list(response.context['messages']):
            msg2.append(message.message)
        self.assertNotEqual(len(msg2), 0, "Erreur lors de l'enregistrement")
        self.assertIn(self.messageProjet, msg2)
        
        # Structure agri bio avec projet et date sans avertissement        
        data['projetbio_set-0-date_engagement_oc'] = '2000-01-01'
        response = self.client.post('/private/contacts/structure/add/', 
                                    data,
                                    follow=True)
        msg2 = list()
        for message in list(response.context['messages']):
            msg2.append(message.message)
        self.assertNotEqual(len(msg2), 0, "Erreur lors de l'enregistrement")
        self.assertNotIn(self.messageProjet, msg2)

# EOF
