# -*- coding: utf-8 -*-
from django.contrib.auth import models as am
from django.core.exceptions import ValidationError
from django.test import TestCase

from contacts import models as cm
from contacts.tests.utils import mfty, cfty
from references import models as rm

from django.contrib import messages
from django.test.client import Client

from contacts import settings_messages as MSG

class ValRdvMessage(TestCase):

    def setUp(self):
        self.messagePersonne = MSG.AVERTISSEMENT_MESSAGE["rdv_collec_sans_part"]
        self.titre = rm.Formation(designation = "test")
        self.titre.save()
        self.usr1 = am.User.objects.create_superuser('usr1', '', 'mdp')
        self.fjur = rm.FormeJuridique()
        self.fjur.save()
        self.i_pm = {
            'date_creation': '2014-10-01',
            'date_derniere_modification': '2014-10-02',
            'utilisateur_derniere_modification': self.usr1,
            'f_juridique': self.fjur,
            }
        self.pm = cfty(cm.Structure, self.i_pm)
        self.i_p = {
            'nom': 'testnom',
            'prenom': 'testprenom',
            'date_naissance': u'2000-01-01',
            'utilisateur_derniere_modification': self.usr1,
            'structure': self.pm,
            }
        self.p = cfty(cm.Personne, self.i_p)
        self.data_rdv = {
            'technicien': u'1', 
            'date': '2014-10-14',
            'rdvpersonne_set-TOTAL_FORMS': u'0', 
            'rdvpersonne_set-INITIAL_FORMS': u'0', 
            'rdvpersonne_set-MAX_NUM_FORMS': u'',
            
            'rdvpersonnemorale_set-TOTAL_FORMS': u'0', 
            'rdvpersonnemorale_set-INITIAL_FORMS': u'0', 
            'rdvpersonnemorale_set-MAX_NUM_FORMS': u'', 
            
            'documentjoint_set-TOTAL_FORMS': u'0', 
            'documentjoint_set-INITIAL_FORMS': u'0', 
            'documentjoint_set-MAX_NUM_FORMS': u'', 
            }
            
    def test_rdv_collec(self):
        """ Un Rdv indiv doit avoir au moins 1 participant """
        # Rdv indiv sans participant avec message d'avertissement
        self.client.login(username = self.usr1.username, password = 'mdp')
        data_rdv = self.data_rdv.copy()
        data_rdv['est_confirme'] = True
        data_rdv['type_rdv'] = u'evt'
        data_rdv['titre_evt'] = u'1'
        response = self.client.post('/private/contacts/rdv/add/', 
                                    data_rdv,
                                    follow=True)
        msg = list()
        for message in list(response.context['messages']):
            msg.append(message.message)
        self.assertIn(self.messagePersonne, msg)
        
        # Rdv indiv avec participant sans message d'avertissement
        data_rdv['rdvpersonne_set-TOTAL_FORMS'] = u'1'
        data_rdv['rdvpersonne_set-0-personne_1'] = u'1'
        response = self.client.post('/private/contacts/rdv/add/', 
                                    data_rdv,
                                    follow=True)
        msg = list()
        for message in list(response.context['messages']):
            msg.append(message.message)   
        self.assertNotIn(self.messagePersonne, msg)
# EOF
