# -*- coding: utf-8 -*-
"""
place for simple regresions tests
"""
from django.contrib.auth import models as am
from django.core.exceptions import ValidationError
from django.test import TestCase

from contacts import models as cm
from contacts.tests.utils import mfty, cfty
from references import models as rm


class ValRdv(TestCase):
    "test de validation des rdv"
    def setUp(self):
        self.usr1 = am.User.objects.create_user('usr1')
        self.i_rdv = {'technicien': self.usr1, 'date': '2014-10-14'}

    def test_rdv_autre(self):
        "un rdv de type autre n'a pas de validation particuliere"
        # rdv non confirme sans erreur
        rdv = mfty(cm.Rdv, self.i_rdv, type_rdv='autre')
        rdv.full_clean()
        # rdv confirme de type autre sans erreur
        rdv.est_confirme = True
        rdv.full_clean()

    def test_rdv_collectif(self):
        """s'il est confirme, un rdv collectif:
        - doit avoir au moins 1 titre (designation)"""
        # rdv de type collectif non confirme sans erreur
        rdv = mfty(cm.Rdv, self.i_rdv, type_rdv='evt')
        rdv.full_clean()
        # rdv de type collectif sans titre en erreur
        rdv.est_confirme = True
        self.assertRaises(ValidationError, rdv.full_clean)
        # rdv de type collectif avec titre sans erreur
        rdv.save()
        rdv.titre_evt = rm.Formation()
        rdv.full_clean()

    def test_rdv_ind(self):
        """s'il est confirme un rdv individuel:
        - doit avoir un participant individuel"""
        # rdv de type indiv, non confirme, sans erreur
        rdv = mfty(cm.Rdv, self.i_rdv, type_rdv='visite_annuelle')
        rdv.full_clean()
        # rdv de type indiv, sans participant ind en erreur
        rdv.est_confirme = True
        self.assertRaises(ValidationError, rdv.full_clean)
        rdv.personne_morale = cm.Structure()
        rdv.full_clean()

    def test_rdv_pbio(self):
        """s'il est confirme un rdv pbio:
        - doit avoir un participant individuel
        - doit avoir un projet bio"""
        # rdv de type pbio, non confirme, sans erreur
        rdv = mfty(cm.Rdv, self.i_rdv, type_rdv='etude')
        rdv.full_clean()
        # rdv de type pbio, sans participant ind en erreur
        rdv.est_confirme = True
        self.assertRaises(ValidationError, rdv.full_clean)
        # rdv de type pbio, avec participant et projet bio sans erreur
        rdv.personne_morale = cm.Structure()
        rdv.projet_bio = cm.ProjetBio()
        rdv.full_clean()

# EOF
