# -*- coding: utf-8 -*-
#

from django.db import models

import django.contrib.auth.models as auth_models


### Données ------------------------------------------------------------------

CHOIX_DEPARTEMENTS = (
        ('62', 'Pas-de-Calais'),
        ('59', 'Nord'),
		("01", "Ain"),
		("02", "Aisne"),
		("03", "Allier"),
		("04", "Alpes-de-Haute-Provence"),
		("05", "Hautes-Alpes"),
		("06", "Alpes-Maritimes"),
		("07", "Ardèche"),
		("08", "Ardennes"),
		("09", "Ariège"),
		("10", "Aube"),
		("11", "Aude"),
		("12", "Aveyron"),
		("13", "Bouches-du-Rhône"),
		("14", "Calvados"),
		("15", "Cantal"),
		("16", "Charente"),
		("17", "Charente-Maritime"),
		("18", "Cher"),
		("19", "Corrèze"),
		("2A", "Corse-du-Sud"),
		("2B", "Haute-Corse"),
		("21", "Côte-d'Or"),
		("22", "Côtes-d'Armor"),
		("23", "Creuse"),
		("24", "Dordogne"),
		("25", "Doubs"),
		("26", "Drôme"),
		("27", "Eure"),
		("28", "Eure-et-Loir"),
		("29", "Finistère"),
		("30", "Gard"),
		("31", "Haute-Garonne"),
		("32", "Gers"),
		("33", "Gironde"),
		("34", "Hérault"),
		("35", "Ille-et-Vilaine"),
		("36", "Indre"),
		("37", "Indre-et-Loire"),
		("38", "Isère"),
		("39", "Jura"),
		("40", "Landes"),
		("41", "Loir-et-Cher"),
		("42", "Loire"),
		("43", "Haute-Loire"),
		("44", "Loire-Atlantique"),
		("45", "Loiret"),
		("46", "Lot"),
		("47", "Lot-et-Garonne"),
		("48", "Lozère"),
		("49", "Maine-et-Loire"),
		("50", "Manche"),
		("51", "Marne"),
		("52", "Haute-Marne"),
		("53", "Mayenne"),
		("54", "Meurthe-et-Moselle"),
		("55", "Meuse"),
		("56", "Morbihan"),
		("57", "Moselle"),
		("58", "Nièvre"),
		("59", "Nord"),
		("60", "Oise"),
		("61", "Orne"),
		("62", "Pas-de-Calais"),
		("63", "Puy-de-Dôme"),
		('64', "Pyrénées-Atlantiques"),
		("65", "Hautes-Pyrénées"),
		("66", "Pyrénées-Orientales"),
		("67", "Bas-Rhin"),
		("68", "Haut-Rhin"),
		("69", "Rhône"),
		("70", "Haute-Saône"),
		("71", "Saône-et-Loire"),
		("72", "Sarthe"),
		("73", "Savoie"),
		("74", "Haute-Savoie"),
		("75", "Paris"),
		("76", "Seine-Maritime"),
		("77", "Seine-et-Marne"),
		("78", "Yvelines"),
		("79", "Deux-Sèvres"),
		("80", "Somme"),
		("81", "Tarn"),
		("82", "Tarn-et-Garonne"),
		("83", "Var"),
		("84", "Vaucluse"),
		("85", "Vendée"),
		("86", "Vienne"),
		("87", "Haute-Vienne"),
		("88", "Vosges"),
		("89", "Yonne"),
		("90", "Territoire de Belfort"),
		("91", "Essonne"),
		("92", "Hauts-de-Seine"),
		("93", "Seine-Saint-Denis"),
		("94", "Val-de-Marne"),
		("95", "Val-d'Oise"),
		("971", "Guadeloupe"),
		("972", "Martinique"),
		("973", "Guyane"),
		("974", "La Réunion"),
		("976", "Mayotte"),
		("000", "Étranger"),
        )

CATEGORIES_CULTURE = (
        u"Productions végétales biologiques",
        u"Autres",
        u"Cultures fourragères",
        u"Racines",
        u"Semences et plants",
        u"Légumes",
        u"Céréales et oléoprotagineux",
        u"Maréchage / Légumes bio",
        u"Vergers basse tige bio (fruits de table)",
        u"Vergers haute tige bio (transformation)",
        u"Surfaces en productions végétales spécialisées",
        )
CHOIX_CATEGORIES_CULTURE = [
        ( c.lower(), c.capitalize() ) for c in CATEGORIES_CULTURE
        ]

### Données géographiques -------------------------------------------------


class Zone(models.Model):
    """
    Zone informelle.

    Source:
        Avesnois
        7 Vallées
        Grande région lilloise
        Flandres
        Artois-Cambrésis
        Haut Pays-Boulonnais
        Scarpe-Escaut
    """
    class Meta:
        ordering = ('nom',)

    nom = models.CharField(max_length=255)

    def __unicode__(self):
        return self.nom


class PetiteRegionAgricole(models.Model):
    """
    Région agricole.
    """
    class Meta:
        verbose_name = "petite région agricole"
        verbose_name_plural = "petites régions agricoles"
        ordering = ('nom',)

    code = models.IntegerField(unique=True)
    nom = models.CharField(max_length=255)
    grande_region = models.IntegerField(verbose_name=u"grande région agricole")
    departement = models.CharField(max_length=3,
                    choices=CHOIX_DEPARTEMENTS, verbose_name=u"département")

    def __unicode__(self):
        return u"%s (%s)" % (self.nom, self.code)


class Arrondissement(models.Model):
    """
    Subdivision des départements.
    """
    class Meta:
        unique_together = ('numero', 'departement')
        ordering = ('nom',)

    numero = models.IntegerField()
    nom = models.CharField(max_length=255)
    departement = models.IntegerField()
    region = models.IntegerField()

    def __unicode__(self):
        return self.nom


class Canton(models.Model):
    """
    En milieu urbain, une commune recouvre parfois plusieurs cantons alors
    qu'en milieu rural, un canton comprend plusieurs communes.
    """
    class Meta:
        unique_together = ('numero', 'arrondissement')
        ordering = ('nom',)

    numero = models.IntegerField()
    nom = models.CharField(max_length=255)
    arrondissement = models.ForeignKey(Arrondissement)

    def __unicode__(self):
        return self.nom


class Commune(models.Model):
    """
    Commune reliée à un canton, un arrondissement, une petite région agricole,
    et une zone.
    """
    class Meta:
        ordering = ('nom',)

    nom = models.CharField(max_length=255)
    code_commune = models.CharField("code INSEE", max_length="15", null=True, blank=True)
    code_postal = models.CharField(max_length="15", null=True, blank=True)
    cantons = models.ManyToManyField("Canton", null=True, blank=True)
    arrondissement = models.ForeignKey("Arrondissement")
    petite_region = models.ForeignKey("PetiteRegionAgricole")
    zone = models.ForeignKey("Zone")
    enjeu_eau = models.NullBooleanField()

    def __unicode__(self):
        return "%s (%s)" % (self.nom, self.code_postal)

    def code_et_ville(self):
        """
        Retourne le code postal et la ville sous forme de chaîne de caractères
        (utilisé pour la création de l'adresse).
        """
        return u"%s %s" % (self.code_postal, self.nom.upper())



### Autres données de référence -----------------------------------------------


class DesignationObjet(models.Model):
    """
    Modèle abstrait pour les listes d'objets utilisant un objet désignation.
    """
    class Meta:
        abstract = True
        ordering = ('designation',)

    designation = models.CharField(max_length=250, verbose_name=u"désignation")

    def __unicode__(self):
        return self.designation


class NomObjet(models.Model):
    """
    Modèle abstrait pour les listes d'objets utilisant un objet nom.
    """
    class Meta:
        abstract = True
        ordering = ('nom',)

    nom = models.CharField(max_length=250, verbose_name=u"nom")

    def __unicode__(self):
        return self.nom


class StatutStructure(DesignationObjet):
    """
    Liste des statuts possibles pour une structure:

    Source:
        Association
        EARL
        GAEC
        Individuelle
        SCEA
    """
    class Meta:
        verbose_name = u"statut de structure"
        verbose_name_plural = "statuts de structure"
        ordering = ('designation',)


class FormeJuridique(DesignationObjet):
    "liste des forme juridique"
    class Meta:
        verbose_name = u"Forme Juridique"
        verbose_name_plural = u"Formes Juridiques"
        ordering = ('designation',)


#class CategorieStructure(DesignationObjet):
#    """
#    Liste des types de structure
#
#    Source: aucune
#    """
#    class Meta:
#        verbose_name = "catégorie de structure"
#        verbose_name_plural = "catégories de structure"
#        ordering = ('designation',)


class TypeActivite(DesignationObjet):
    """
    Liste des types d'activité agricole.

    Source:
        100% herbe
        Arboriculture
        Légumes
        Maraîchage
        Polyculture
        Production spécialisée
    """
    class Meta:
        verbose_name = "type d'activité"
        verbose_name_plural = "types d'activité"
        ordering = ('designation',)


class AutreQualificatifExploitation(DesignationObjet):
    """
    Liste des types d'élevages.

    Source:
        Bovins/Lait
        Bovins/Viande
        Ovins
        Chèvres
        Poules pondeuses
        Volailles de choix
        Fruits rouges
        Miel
        Endives
        Escargots
        Basses tiges
        Hautes tiges
        Champignons
    """
    class Meta:
        verbose_name = "autre qualiticatif de l'exploitation"
        verbose_name_plural = "autres qualiticatifs de l'exploitation"
        ordering = ('designation',)


class Laiterie(NomObjet):
    """
    Liste des laiteries.

    Source:
        Biolait
        Catillon
        Danone
        From de Th
        GIE Av Lait
        Laitnaa
        Milcobel
        Nestlé
        Novandie
        Prospé Ferm
        Sodiaal
        Ucanel
        VD
        VPM
    """
    class Meta:
        ordering = ('nom',)


class OrganismeDeCertification(NomObjet):
    """
    Liste des organismes de certification biologique.

    Source:
        Ecocert
        Qualité France
    """
    class Meta:
        verbose_name = "organisme de certification"
        verbose_name_plural = "organismes de certification"
        ordering = ('nom',)


class Etiquette(DesignationObjet):
    """
    """
    class Meta:
        verbose_name = "étiquette"
        verbose_name_plural = "étiquettes"
        ordering = ('designation',)

    description = models.CharField(max_length="255",
                                   null=True, blank=True,
                                   verbose_name=u"description")


class StatutCommercialisation(DesignationObjet):
    """
    Liste des statuts possibles pour les commercialisation en vente directe.
    """
    class Meta:
        verbose_name = "statut commerc. en vente directe"
        verbose_name_plural = "statuts commerc. en vente directe"
        ordering = ('designation',)


class TypeService(DesignationObjet):
    """
    Liste des types de services offerts par les agriculteurs bio.
    (fait partie de l'observatoire).
    """
    class Meta:
        verbose_name = "type de service"
        verbose_name_plural = "types de service"
        ordering = ('designation',)


class Culture(DesignationObjet):
    """
    Liste des cultures (utilisé dans les assolement de DonneesBio).
    """
    class Meta:
        verbose_name = "Culture"
        verbose_name_plural = "Cultures"
        ordering = ('categorie', 'indice_ordre',)
    
    categorie = models.CharField(
        u"catégorie", max_length="100", choices=CHOIX_CATEGORIES_CULTURE)
    indice_ordre = models.IntegerField(
        u"Indice d'ordonnancement", null=True, blank=True,
        help_text=u"Permet d'ordonner les cultures dans leur catégorie")


class ClientCulture(DesignationObjet):
    """
    Liste de clients pour les cultures.
    """
    class Meta:
        verbose_name = "Client de culture"
        verbose_name_plural = "Clients de culture"
        ordering = ('designation',)


class Abattoir(NomObjet):
    """
    Abbatoirs.
    """
    class Meta:
        ordering = ('nom',)


class AcheteurOeufs(DesignationObjet):
    """
    Liste de clients pour les cultures.
    """
    class Meta:
        verbose_name = "Acheteur d'œufs"
        verbose_name_plural = "Acheteurs d'œufs"
        ordering = ('designation',)


class Formation(DesignationObjet):
    """
    Liste des formations suivies par les agriculteurs.
    """
    class Meta:
        verbose_name = "Formation"
        verbose_name_plural = "Formations"
        ordering = ('designation',)


class TypeFormation(DesignationObjet):
    """
    Liste des types de formation que les agriculteurs peuvent vouloir suivre.
    """
    class Meta:
        verbose_name = "Désignation Évenement"
        verbose_name_plural = "Désignations Évenements"
        ordering = ('designation',)


class FinanceProg(DesignationObjet):
    "Liste des financement/programme concerné"
    class Meta:
        verbose_name = u"Financement/Programme"
        verbose_name_plural = u"Financements/Programmes"
        ordering = ('designation',)


class PublicCible(NomObjet):
    "public cible"
    class Meta:
        verbose_name = "Public Cible"
        verbose_name_plural = "Publics Cible"
        ordering = ('nom',)

# EOF
